@@ -1,7 +1,7 @@
 package Complete::Getopt::Long;
 
-our $DATE = '2014-11-10'; # DATE
-our $VERSION = '0.15'; # VERSION
+our $DATE = '2014-11-18'; # DATE
+our $VERSION = '0.16'; # VERSION
 
 use 5.010001;
 use strict;
@@ -18,26 +18,75 @@ our %SPEC;
 sub _default_completion {
     my %args = @_;
     my $word = $args{word} // '';
+
+    # try completing '$...' with shell variables
     if ($word =~ /\A\$/) {
-        return {completion=>
-                    Complete::Util::complete_env(word=>$word, ci=>$args{ci}),
-                escmode=>'shellvar'};
+        require Complete::Util;
+        {
+            my $compres = Complete::Util::complete_env(
+                word=>$word, ci=>$args{ci});
+            last unless @$compres;
+            return {completion=>$compres, escmode=>'shellvar'};
+        }
+        # if empty, fallback to searching file
     }
-    if ($word =~ /\A~/) {
-        eval { require Complete::Unix };
-        return [] if $@;
-        $word =~ s/\A~//;
-        return [
-            map {"~$_"}
-                @{ Complete::Unix::complete_user(word=>$word, ci=>$args{ci}) }
-        ];
+
+    # try completing '~foo' with user dir (appending / if user's home exists)
+    if ($word =~ m!\A~([^/]*)\z!) {
+        {
+            eval { require Unix::Passwd::File };
+            last if $@;
+            my $res = Unix::Passwd::File::list_users(detail=>1);
+            last unless $res->[0] == 200;
+            my $compres = Complete::Util::complete_array(
+                array=>[map {"~" . $_->{user} . ((-d $_->{home}) ? "/":"")}
+                            @{ $res->[2] }],
+                word=>$word, ci=>$args{ci},
+            );
+            last unless @$compres;
+            return {completion=>$compres, path_sep=>'/'};
+        }
+        # if empty, fallback to searching file
     }
+
+    # try completion '~foo/blah' as if completing file, but do not expand ~foo
+    if ($word =~ m!\A(~[^/]*)/!) {
+        {
+            my $tilde = $1;
+            my $dir = [glob($tilde)]; # glob will expand ~foo to /home/foo
+            last unless @$dir;
+            # XXX unlike in bash, ~foo/<tab> will not match dotfiles if we use
+            # '*'
+            my $compres = [glob("$word*")];
+            last unless @$compres;
+            # unexpand ~foo
+            for (@$compres) {
+                $_ .= "/" if (-d $_);
+                s/\A\Q$dir->[0]\E/$tilde/;
+            }
+            return {completion=>$compres, path_sep=>'/'};
+        }
+        # if empty, fallback to searching file
+    }
+
+    # try completing something that contains wildcard with glob. for
+    # convenience, we add '*' at the end so that when user type [AB] it is
+    # treated like [AB]*.
     require String::Wildcard::Bash;
     if (String::Wildcard::Bash::contains_wildcard($word)) {
-        return {completion=>[glob($word)], path_sep=>'/'};
+        {
+            my $compres = [glob("$word*")];
+            last unless @$compres;
+            for (@$compres) {
+                $_ .= "/" if (-d $_);
+            }
+            return {completion=>$compres, path_sep=>'/'};
+        }
+        # if empty, fallback to searching file
     }
-    return {completion=>Complete::Util::complete_file(word=>$word), path_sep=>'/'};
-};
+    return {completion=>Complete::Util::complete_file(word=>$word),
+            path_sep=>'/'};
+}
 
 # return the key/element if $opt matches exactly a key/element in $opts (which
 # can be an array/hash) OR expands unambiguously to exactly one key/element in
@@ -78,6 +127,13 @@ This routine can complete option names, where the option names are retrieved
 from `Getopt::Long` specification. If you provide completion routine in
 `completion`, you can also complete _option values_ and _arguments_.
 
+Note that this routine does not use `Getopt::Long` (it does its own parsing) and
+currently is not affected by Getopt::Long's configuration. Its behavior mimics
+Getopt::Long under these configuration: `bundling`, `no_ignore_case`. Which I
+think is the sensible default. This routine also does not currently support
+`auto_help` and `auto_version`, so you'll need to add those options specifically
+if you want to recognize `--help/-?` and `--version`, respectively.
+
 _
     args => {
         getopt_spec => {
@@ -470,7 +526,7 @@ Complete::Getopt::Long - Complete command-line argument using Getopt::Long speci
 
 =head1 VERSION
 
-This document describes version 0.15 of Complete::Getopt::Long (from Perl distribution Complete-Getopt-Long), released on 2014-11-10.
+This document describes version 0.16 of Complete::Getopt::Long (from Perl distribution Complete-Getopt-Long), released on 2014-11-18.
 
 =head1 SYNOPSIS
 
@@ -489,6 +545,13 @@ This routine can complete option names, where the option names are retrieved
 from C<Getopt::Long> specification. If you provide completion routine in
 C<completion>, you can also complete I<option values> and I<arguments>.
 
+Note that this routine does not use C<Getopt::Long> (it does its own parsing) and
+currently is not affected by Getopt::Long's configuration. Its behavior mimics
+Getopt::Long under these configuration: C<bundling>, C<no_ignore_case>. Which I
+think is the sensible default. This routine also does not currently support
+C<auto_help> and C<auto_version>, so you'll need to add those options specifically
+if you want to recognize C<--help/-?> and C<--version>, respectively.
+
 Arguments ('*' denotes required arguments):
 
 =over 4
@@ -598,7 +661,7 @@ Please visit the project's homepage at L<https://metacpan.org/release/Complete-G
 
 =head1 SOURCE
 
-Source repository is at L<https://github.com/sharyanto/perl-Complete-Getopt-Long>.
+Source repository is at L<https://github.com/perlancar/perl-Complete-Getopt-Long>.
 
 =head1 BUGS
 
